﻿using System.Collections.Generic;
using System.Linq;
using System;
using Proteus.Domain.Foundation;

namespace AltNetSample.Domain
{
    public class Customer : IdentityPersistenceBase<Customer, int>
    {
        private Name _name;


        private Address _address;


        private IList<Order> _orders;

        /// <summary>
        /// Initializes a new instance of the Customer class.
        /// </summary>
        public Customer()
        {
            _orders = new List<Order>();
        }

        public Address Address
        {
            get { return _address; }
        }
        public string Firstname
        {
            get { return _name.Firstname; }
        }

        public int Id
        {
            get { return _persistenceId; }
        }

        public string Lastname
        {
            get { return _name.Lastname; }
        }

        public IEnumerable<Order> OrdersNotCancelled
        {
            get { return _orders.Where(o => o.Status != OrderStatus.Cancelled).AsEnumerable<Order>(); }
        }

        public void AddOrder(Order order)
        {
            /*
             * _NOT_
             * if (order.Items.Count() == 0)
             *   throw new AddOrderOperationException("An order cannot be added without having any Items!");
             *   
             * Customer class should have no knowledge of how an order determines its valid to be added!
             */

            if (order.CanBeAdded)
                _orders.Add(order);
            else
                /*
                 *the thing _performing_ the action decides how to react when can't do the op 
                 * and _NOT_ the thing being asked if its OK
                 */
                throw new AddOrderOperationFailure("Could not add order!");
        }

        public void CancelOrder(Order order)
        {
            //we can use .Contains in the next statement b/c of identity-value-equality!
            if (_orders.Contains(order))
            {
                /*
                 * because of identity-value-equality ensured among entities,
                 * we don't have to write this next statement as...
                 * var orderToCancel = _orders.Where(o => o.Id == order.Id).Single();
                 */
                var orderToCancel = _orders.Where(o => o == order).Single();

                /*
                 * HOLLYWOOD principle: tell, don't ask!
                 * _NOT_ orderToCancel.Status = OrderStatus.Cancelled;
                 */
                orderToCancel.Cancel();
            }
        }

        public void ChangeName(string firstname, string lastname)
        {
            ChangeName(new Name(firstname, lastname));
        }

        public void ChangeName(Name name)
        {
            if (new CustomerNameChangeRule().IsSatisfiedBy(name))
                _name = name;
            else
                throw new NameChangeOperationFailure("Cannot change name!");
        }

        public void ChangeAddress(Address newAddress)
        {
            if (new CustomerAddressChangeRule().IsSatisfiedBy(newAddress))
                _address = newAddress;
            else
                throw new AddressChangeOperationFailure("Cannot change name!");
            
        }

    }
}